"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _graphqlQueryCompress = _interopRequireDefault(require("graphql-query-compress"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const GRAPHQL_TOKEN_START = /^[a-z]/i;
const GRAPHQL_TOKEN_END = /[a-z0-9]$/i;
const DEFAULT_COMMENT = "graphql";

function _default() {
  /**
   * Checks if a `TaggedTemplateExpression` is tagged with a function named with the
   * tagName setting, 'gql' by default.
   * If a match is found the contents of the TemplateLiteral is compressed.
   * If the tag has no existing binding it is replaced with the provided tagFunction
   * or it is removed.
   */
  function compressTaggedTemplateExpression(path, state) {
    const _path$node = path.node,
          tag = _path$node.tag,
          quasi = _path$node.quasi;
    const _state$opts = state.opts,
          _state$opts$tagName = _state$opts.tagName,
          tagName = _state$opts$tagName === void 0 ? "gql" : _state$opts$tagName,
          tagFunction = _state$opts.tagFunction;

    if (tag.name === tagName) {
      // the quasi property contains the TemplateLiteral
      compressTemplateLiteral(quasi);

      if (!path.scope.hasBinding(tagName)) {
        if (tagFunction) {
          // we have an actual tag function, update the name to match it
          tag.name = tagFunction;
        } else {
          // tag function is empty, replace the TaggedTemplateExpression
          // with the nested TemplateLiteral.
          path.replaceWith(quasi);
        }
      }
    }
  }
  /**
   * Checks if the `TemplateLiteral` or `StringLiteral` has a leading comment that
   * matches the comment setting (case-insensitive), 'graphql' by default.
   *
   * If a matching literal with comment is found the literal is compressed.
   * When not disabled by the setting removeComments, the matched comment is removed.
   */


  function compressCommentedNode(path, state) {
    if (!path.node.leadingComments) {
      return;
    }

    const _path$node2 = path.node,
          leadingComments = _path$node2.leadingComments,
          type = _path$node2.type;
    const commentTrigger = (state.opts.comment || DEFAULT_COMMENT).toLowerCase();
    const commentIndex = leadingComments.findIndex(comment => trimComment(comment.value).toLowerCase() === commentTrigger);

    if (commentIndex >= 0) {
      // there is a leading comment that matches our 'comment' setting
      switch (type) {
        case "TemplateLiteral":
          compressTemplateLiteral(path.node);
          break;

        case "StringLiteral":
          compressStringLiteral(path.node);
          break;
      } // remove the matched comment


      if (state.opts.removeComments !== false) {
        path.node.leadingComments.splice(commentIndex, 1);
      }
    }
  }
  /**
   * Apply the `compress` function to the given `TemplateLiteral`.
   * @param {{quasis: {value: {raw: string, cooked: string}}[], type: string}} node
   */


  function compressTemplateLiteral(node) {
    node.quasis.forEach(({
      value,
      tail
    }, index) => {
      const part = (0, _graphqlQueryCompress.default)(value.raw).trim(); // Multiple quasi's should be separated as they have variables next to them. (#4, #5)

      const prefix = index > 0 && (GRAPHQL_TOKEN_START.test(part) || part === "") ? " " : "";
      const postfix = !tail && GRAPHQL_TOKEN_END.test(part) ? " " : "";
      const compressed = prefix + part + postfix;
      value.raw = compressed;
      value.cooked = compressed;
    });
  }
  /**
   * Apply the `compress` function to the given `StringLiteral`.
   * @param node
   */


  function compressStringLiteral(node) {
    node.value = (0, _graphqlQueryCompress.default)(node.value).trim();
  }
  /**
   * Trim excess white space and '*' characters from the given comment.
   * @param {string} comment The given comment.
   * @returns {string} The trimmed comment.
   */


  function trimComment(comment) {
    return comment.replace(/^[\s*]+/, "").replace(/[\s*]+$/, "");
  }

  return {
    name: "transform-compress-graphql",
    visitor: {
      TaggedTemplateExpression: compressTaggedTemplateExpression,
      TemplateLiteral: compressCommentedNode,
      StringLiteral: compressCommentedNode
    }
  };
}